import curses

class CursesDisplay():
    def __init__(self):
        self.stdscr = curses.initscr()
        
        begin_x = 0
        begin_y = 0
        height = 25
        width = 0
        self.status_window = curses.newwin(height, width, begin_y, begin_x)
        begin_y += height
        height = 100
        self.debug_window = curses.newwin(height, width, begin_y, begin_x)
            
    def end(self):
        curses.nocbreak()
        self.status_window.keypad(0)
        curses.echo()
        curses.endwin()

    def clear(self):
        self.status_window.clear()
        self.debug_window.clear()

    def finish(self):
        self.status_window.refresh()
        self.debug_window.refresh()

    def debugprint(self, *args):
        info = " ".join(map(lambda x: str(x), args)) + "\n"
        self.status_window.clear()
        self.debug_window.addstr(info)

    def errorprint(self, *args):        
        error = " ".join(map(lambda x: str(x), args)) + "\n"
        self.status_window.clear()
        self.status_window.addstr(error)




"""This module is intended for tracing code and doing 'post mortem' inspection of objects.
Typical usage is
  - create Tracer object 'tracer'
  - tracer.install()
  - RUN YOUR CODE
  - tracer.uninstall()
  - tracer.inspect(file, line)

KNOWN ISSUE : RUN YOUR CODE cannot be at the same level than install / uninstall, it must be in a function call. I don't know why yet.
"""

import sys
import inspect
import copy
import time
import traceback

class Tracer(object):
    def __init__(self, file_white_list = None):
        self.run_info = {}
        self.last_trace_data_stack = [{}]
        self.file_white_list = file_white_list

    def istracing(self, filename):
        return filename in self.file_white_list

    def create_new_run_info(self, file, line, data, dedupCheck):
        # Create the record for this file / line
        if file not in self.run_info:
            self.run_info[file] = {}
        if line not in self.run_info[file]:
            self.run_info[file][line] = []
        if not dedupCheck or len(self.run_info[file][line]) == 0:
            # Record time, and append the information to the set of track (getting through a line may happen several time !)
            self.run_info[file][line] += [(time.time() - self.start_time, data)]
        # Record the last data for comparison at next step
        self.last_trace_data_stack[-1] = data

    def print_exception(self, msg):
        return
        print msg, "\n", traceback.format_exc()

    def object_repr(self, object):
        return repr(object)

    def trace_frame(self, frame, event, arg):
        """Trace a single frame.
        This try to determine which var changed, and at which time.
        This then allows to sort the variables by younger change to older one."""
        if event.startswith("c_"):
            return self.trace_frame

        if event == "call":
            # Push empty last_trace data
            self.last_trace_data_stack += [{}]


        try:
            # Get the current file name and current line number
            filename = inspect.getfile(frame)

            if self.file_white_list != None:
                if filename not in self.file_white_list:
                    return self.trace_frame

            linenumber = frame.f_lineno
            # Get the right trace_data
            last_trace_data = self.last_trace_data_stack[-1]

            # Compare the locals with the previous ones
            new_trace_data = {}

            # Enumerate the locals
            for k, v in frame.f_locals.iteritems():
                try:
                    # Check if the values are the same as in the previous frame info
                    same = False

                    # Check if i
                    if k in last_trace_data:
                        # You may try different object representation, that explains the enumeration
                        for vv in [self.object_repr(v)]:
                            try:
                                # Check if last data is the same (object at index 0 is "framestamp")
                                if last_trace_data[k][1] == vv:
                                    same = True
                                break
                            except:
                                self.print_exception("CASE 1")

                    if not same:
                        # Not same: store the new version
                        for vv in [self.object_repr(v)]:
                            try:
                                # Store the record ("framestamp", data)
                                new_trace_data[k] = [self.frame_counter, vv]
                                # Increment frame_counter
                                self.frame_counter += 1
                            except:
                                self.print_exception("CASE 2")
                    else:
                        # It's the same : keep the last ("framestamp", data) record
                        new_trace_data[k] = last_trace_data[k]

                except:
                    self.print_exception("CASE 3")

            # Create the record for this frame
            self.create_new_run_info(filename, linenumber, new_trace_data, event == "return")
        except:
            self.print_exception("CASE 4")

        if event == "return":
            self.last_trace_data_stack = self.last_trace_data_stack[:-1]

        return self.trace_frame

    def install(self):
        """Install the tracer"""
        self.start_time = time.time()
        self.frame_counter = 0
        sys.settrace(self.trace_frame)

    def uninstall(self):
        """Uninstall the tracer"""
        sys.settrace(None)

    def inspect(self, file, line):
        """Inspect a single line in a file"""

        for i in range(0, 10):
            ret = self.run_info.get(file, {}).get(line + i)
            if ret:
                return ret

        return ret

    def inspect_all(self):
        """Dump all the information stored"""
        for k,v in self.run_info.iteritems():
            print "file=", k
            keys = v.keys()
            keys.sort()

            for k in keys:
                print "line = %s, data =%s" % (k, v[k])



import gevent
import jarvis.server.utils as utils
import jarvis.server.broker as broker
import jarvis.server.redisconnect as redisconnect
import jarvis.redis.dict as redis_dict
import jarvis.server.state as state
import json

class BaseStateObserver(gevent.Greenlet):
    def __init__(self, session, user, queue, max_messages = None):
        gevent.Greenlet.__init__(self)
        self.session = session
        self.user = user
        self.queue = queue
        self.message_count = 0
        self.max_messages = max_messages

        self.wake_up_channel = "-".join([user, session if session else self.random_session(), "wakeup"])
        self.ready = False

    def random_session(self):
        return utils.generate_random(10)

    def emit(self, msg):
        self.message_count += 1

        if self.max_messages < 0:
            ret = True
        else:
            ret = self.message_count < self.max_messages

        if not ret:
            self.queue.put(StopIteration)
        return ret

    def add_id(self, id):
        if id == '':
            raise Exception("Empty id")

        while not self.ready:
            gevent.sleep(0.01)

        brok = broker.Broker()
        publisher = brok.publisher()
        publisher.publish(self.wake_up_channel, {"id":id, "op":"listen"})

    def prepare(self):
        pass

    def _run(self):
        # Create redis client
        client = redisconnect.connect()
        self.dict = redis_dict.RedisDict(client)
        # Create the broker
        self.state = state.State(self.user, client)
        self.subscriber = self.state.subscriber()
        self.lastTime = {}

        # Subscribe to the "wake-up" signal
        self.subscriber.subscribe(self.wake_up_channel)
        self.ready = True

        self.prepare()

        try:
            # Create top level redis dict
            for operation in self.subscriber:
                operation = operation[1]
                id = operation["id"]
                if operation.get("op") == "listen":
                    self.subscriber.subscribe(self.state.id_to_key(id))

                    try:
                        operation = self.state.get_agregated_operation(id)
                    except KeyError:
                        # We will only get subscribe messages, as the object does exist yet
                        continue

                    self.emit(operation)
                    self.lastTime[id] = operation["time"]
                else:
                    t = operation["time"]

                    if id in self.lastTime and t > self.lastTime[id]:
                        go_on = self.emit(operation)
                        self.lastTime[id] = t
                    if not go_on:
                        break
        finally:
            self.queue.put(StopIteration)



class JsonStateObserver(BaseStateObserver):
    def emit(self, msg):
        self.queue.put(json.dumps(msg) + "\n")
        return super(JsonStateObserver, self).emit(msg)

class EventSourceStateObserver(BaseStateObserver):
    def emit_(self, type, content, final_data = False):
        suffix = "\n\n" if final_data else "\n"
        output = type + ": " + content + suffix
        self.queue.put(output)

    def emit(self, msg):
        self.emit_("data", json.dumps(msg), final_data = True)

        return super(EventSourceStateObserver, self).emit(msg)

    def prepare(self):
        # 2kb padding for IE
        self.emit_('', ' ' * 2048)
        self.emit_('retry', '2000')



import sys
from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4.QtCore import Qt
import os.path
import codecs
import osgqt
import osgDB
import osg
import shutil
import jarvis
import traceback

class MyTextEdit(QtGui.QTextEdit):
    def __init__(self, text, father):
        super(MyTextEdit, self).__init__(text, father)
#        self.setFontPointSize(20)
#    def keyPressEvent(self, event):
#        print event

class JarvisMain(QtGui.QWidget):

    def __init__(self, layout=None):
        self.osg_enable = True
        super(JarvisMain, self).__init__()

        self.initUI(layout)

    def message(self, object):
        try:
            fun_name = object["fun"]
            args = object.get("args", [])
            kwargs = object.get("kwargs", {})
            getattr(self, fun_name)(*args, **kwargs)
        except:
            print traceback.format_exc()
            pass

    def start(self):
        self.debug.clear()
        self.error.clear()

    def finish(self):
        pass

    def display(self):
        print self.editor.toPlainText()

#    def wheelEvent(self, event):
#        print self.editor.setFocus(False)

#    def keyPressEvent(self, event):
#         print event

    def createEditor(self, text, width, height):
        editor = MyTextEdit(text, self)
        editor.setMinimumWidth(width)
        editor.setMinimumHeight(height)
        return editor

    def createOSG(self, width, height):
        osgWidget = osgqt.PyQtOSGWidget(self)
        osgWidget.setMinimumWidth(width)
        osgWidget.setMinimumHeight(height)
        return osgWidget

    def parse_layout(self, layout):
        ret = {}
        f = open(layout)
        for line in f:
            parts = line.split("=")
            ret[parts[0]] = int(parts[1])
        return ret

    def initUI(self, layout=None):
        self.setWindowTitle('Jarvis')

        if layout is not None:
            layout = self.parse_layout(layout)
        else:
            layout = {}

        WIDTH=480
        HEIGHT=1024
        self.setGeometry(1680 - WIDTH, 0, WIDTH, 1050)

#        self.setWindowIcon(QtGui.QIcon('web.png'))

        text = ""
#        for i in range(10):
#            text += '<b>Text%d</b><br/>' % i

        self.topBox = QtGui.QHBoxLayout(self)
        self.topBox.setContentsMargins(0,0,0,0)
        self.topBox.setSpacing(0)
        self.topBox.setGeometry(QtCore.QRect(0, 0, WIDTH, HEIGHT))

#        self.editor = self.createEditor(text, 300, 600)
#        self.editor.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
#        self.topBox.addWidget(self.editor)

        self.rightBox = QtGui.QVBoxLayout()
        self.rightBox.setSpacing(0)
        self.topBox.addLayout(self.rightBox)


        self.error = self.createEditor("", WIDTH, 200)
        self.debug = self.createEditor("", WIDTH, 200)

        if self.osg_enable:
            self.osgView = self.createOSG(WIDTH, int(WIDTH * 9 / 16.0))

        self.rightBox.addWidget(self.error, 0, Qt.AlignRight)
        self.rightBox.addWidget(self.debug, 0, Qt.AlignRight)
        if self.osg_enable:
            self.rightBox.addWidget(self.osgView, 0, Qt.AlignRight)

#        editor.textChanged.connect(self.display)
#        editor.wheelEvent.connect(self.wheelEvent)
#        self.editor.setFocus()
        self.filename = None
        self.show()

    def atomic_write(self, filename, text):
        f = open(filename + ".tmp", "w")
        f.write(text)
        f.close()
        shutil.move(filename + ".tmp", filename)

    def debugprint(self, *args):
        text = " ".join(map(lambda x: str(x), args))
        self.debug.append(text)
        text = self.debug.toPlainText()

        debug_file = jarvis.get_filename(jarvis.DEBUG_FILE)
        self.atomic_write(debug_file, text + "\n")

    def errorprint(self, *args):
        text = " ".join(map(lambda x: str(x), args))
        self.error.append(text)
        text = self.error.toPlainText()

        error_file = jarvis.get_filename(jarvis.ERROR_FILE)
        self.atomic_write(error_file, text + "\n")

    def reset(self):
        self.debug.setText("")
        self.error.setText("")
        self.osgView.resetSceneData(None)

    def osgprint(self, data):
        self.osgView.setSceneData(data)

    def audioemit(self, data, skip = 0.0):
        self.osgView.setAudioData(data, skip)

    def setlooptime(self, loopTime):
        self.osgView.setLoopTime(loopTime)

    def runcommand(self, fun):
        fun()

    def getosgviewer(self):
        return self.osgView.getosgviewer()

    def file_dialog(self):
        fd = QtGui.QFileDialog(self)
        filename = fd.getOpenFileName()
        if os.path.isfile(filename):
            text = open(filename).read()
            self.editor.setText(text)
            self.filename = filename

            s = codecs.open(self.filename,'w','utf-8')
            s.write(unicode(self.ui.editor_window.toPlainText()))
            s.close()



import jarvis.redis.base as redisbase
import jarvis.server.redisconnect as redisconnect
import jarvis.redis.pubsub as pubsub

class Broker(redisbase.RedisBase):
    def __init__(self, client = None):
        self.client = client

    def connect(self):
        if self.client == None:
            self.client = redisconnect.connect()
        return self.client

    def subscriber(self):
        client = self.connect()
        return pubsub.RedisJsonSubscriber(client)

    def publisher(self):
        client = self.connect()
        publisher = pubsub.RedisJsonPublisher(client)
        return publisher




import os
import argparse
import traceback

ml = None

# This is the place where is stored the currently tested 'filename:function' .
TEST_FILENAME_FUNCTION="test_filename_function.txt"

# This is the file where errors produced by calls to jarvis.command.error are written.
ERROR_FILE="error.txt"
DEBUG_FILE="debug.txt"
INSPECT_VAR_QUERY="inspect_var_query.txt"
INSPECT_VAR_QUERY_RESPONSE="inspect_var_query_response.txt"

def get_home():
    jarvis_home = os.getenv("JARVIS_HOME", "/tmp/jarvis")
    try:
        os.makedirs(jarvis_home)
    except OSError, e:
        if e.errno != 17:
            raise

    return jarvis_home

def get_filename(key):
    error_file = os.path.join(get_home(), key)
    return error_file

def run(filename_function, layout=None):
    global ml
    import qtdisplay
    import mainloop

    try:
        ml = mainloop.MainLoop(filename_function)

        display = qtdisplay.QTDisplay(ml, layout)
        ml.setdisplay(display)

        display.init()

    except KeyboardInterrupt:
        pass
    except Exception, e:
        print traceback.format_exc(e)
    finally:
        display.destroy()



import code, traceback, signal

def debug(sig, frame):
    """Interrupt running process, and provide a python prompt for
    interactive debugging."""
    d={'_frame':frame}         # Allow access to frame object.
    d.update(frame.f_globals)  # Unless shadowed by global
    d.update(frame.f_locals)

    i = code.InteractiveConsole(d)
    message  = "Signal recieved : entering python shell.\nTraceback:\n"
    message += ''.join(traceback.format_stack(frame))
    i.interact(message)

def listen():
    signal.signal(signal.SIGUSR1, debug)  # Register handler

def main():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument("--nogl", default=False, action="store_true", help="disable gl window")
    parser.add_argument("--filename_function", metavar="NAME")
    parser.add_argument("--layout", metavar="LAYOUT", default=None)

    args = parser.parse_args()
    run(args.filename_function, args.layout)

if __name__ == "__main__":
    listen()
    main()





import sys
import qt
from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4 import QtOpenGL

# Used to wrap calls to QTDisplay missing functions, to proxy it to the JarvisMain Qt Widget
class CallWrapper:
    def __init__(self, parent, name):
        self.parent = parent
        self.name = name

    def __call__(self, *args, **kwargs):
        self.parent.message(self.name, *args, **kwargs)
        
class QTDisplay(QtCore.QObject):
    """This class act as a proxy between the rest of the world and the actual qt widgets"""

    # Message signal handler : it's used to pass all messages to Qt widgets
    message_available = QtCore.pyqtSignal(object)
        
    def __init__(self, mainloop, layout=None):
        # Initialize this as as Qt object
        QtCore.QObject.__init__(self)
        # Create the application
        self.app = QtGui.QApplication(sys.argv)
        
        # Setup OpenGL to use multisampling for antialiasing
        glf = QtOpenGL.QGLFormat.defaultFormat()
        glf.setSampleBuffers(True)
        glf.setSamples(4)
        QtOpenGL.QGLFormat.setDefaultFormat(glf)

        # Create the main widget
        self.jarvismain = qt.JarvisMain(layout)
#        self.jm.raise_()

        # Connect the message to the main widget
        self.message_available.connect(self.jarvismain.message)

        # Start the loop once the Qt App was created
        mainloop.start()

    # Initalize : launch the app
    def init(self):
        sys.exit(self.app.exec_())

    # Destroy the app
    def destroy(self):
        pass

    # Get the inner osg viewer
    def getosgviewer(self):
        return self.jarvismain.getosgviewer()

    # For all other display function calls, act as a proxy to qt for calls
    def message(self, fun, *args, **kwargs):
        self.message_available.emit({"fun":fun, "args":args, "kwargs":kwargs})

    # Catch the missing function calls
    def __getattr__(self, name):
        return CallWrapper(self, name)
                



import jarvis.redis.dict as redisdict
import jarvis.server.broker as broker
import json
import copy
import time

# The State store objects and receive/send operations  on objects through a broker 

STATE_PREFIX = "state"
KEY_SEPARATOR = "-"

class StateObject(object):    
    def __init__(self, state, id, user, type, full_object):
        self.state = state
        self.id = id
        self.user = user
        self.type = type
        self.load(full_object)

    def load(self, full_object):
        self.full_object = full_object
        self.value = self.decode_value()

    def decode_value(self):
        return self.full_object.get("value", None)
    
    def encode_value(self):
        self.full_object["value"] = self.value

    def create_operation(self, op, timestamp = None, operation_parameters = None):
        operation = {}
        operation["op"] = op
        operation["user"] = self.user
        operation["id"] = self.id
        operation["time"] = timestamp or self.full_object.get("time")
        operation["parameters"] = operation_parameters or {}
        return operation
                
    def save(self, pipe = None):
        timestamp = time.time()
        self.full_object["time"] = timestamp
        self.encode_value()
        self.state[self.id] = self.full_object
        
        return timestamp

    def op(self, op, **kwargs):
        objFunction = getattr(self, op + "_operation")
        self.state.op(op, self, objFunction, **kwargs)

    def get_agregated_operation(self):
        return self.create_operation("set", None, {"content":self.value})
        
class StringObject(StateObject):
    def set_operation(self, content):
        self.value = content
        return self.create_operation("set", None, {"content":content})

    def append_operation(self, content):
        self.value += content
        return self.create_operation("append", None, {"content":content})


class StateObjectSubscriber(object):
    def __init__(self, state):
        self.state = state
        self.subscriber = state.broker.subscriber()

    def subscribe(self, id):
        key = self.state.id_to_key(id)
        self.subscriber.subscribe(id)

    def __iter__(self):
        """
        Yield ``(channel, message)`` tuples as messages arrive on subscribed
        channels.
        """
        for channel, item in self.subscriber:
            yield channel, item            

class State(redisdict.RedisDict):
    def __init__(self, user, client):
        self.user = user

        assert(client != None)
        
        # Create the broker
        self.broker = broker.Broker(client)
        self.publisher = self.broker.publisher()

        super(State, self).__init__(client)

    def id_to_key(self, id):
        key = KEY_SEPARATOR.join([STATE_PREFIX, self.user, id])

        return key
        
    def key_to_id(self, key):
        return key[len(STATE_PREFIX):]
                
    def factory(self, type, id, full_object):
        type = type.capitalize()
        constructor = globals()[type + "Object"]
        o = constructor(self, id, self.user, type, full_object)
        return o

    def __setitem__(self, id, value):
        key = self.id_to_key(id)
        super(State, self).__setitem__(key, json.dumps(value))
        
    def __getitem__(self, id):
        key = self.id_to_key(id)
        value = super(State, self).__getitem__(key)
        full_object = json.loads(value)
        type = full_object.get("type")
        return self.factory(type, id, full_object)
    
    def get_or_create(self, id, type):
        try:
            return self[id]
        except:
            full_object = {"id": id, "user": self.user, "type": type}
            return self.factory(type, id, full_object)

    def get_agregated_operation(self, id):
        obj = self[id]
        return obj.get_agregated_operation()
        
    def publish(self, id, message):
        key = self.id_to_key(id)
        self.publisher.publish(key, message)
        
    def transaction(self, function, others, watches):
        others += [self.publisher, self.broker]

        tr_watches = []
        for k in watches:
            tr_watches = self.id_to_key(k)
            
        super(State, self).transaction(function, others = others, watches = tr_watches)
        
    def op(self, op, obj, objFunction, **kwargs):
        def update_state_set(pipe):
            self.get_or_create(obj.id, obj.type)
            pipe.multi()
            # Set the value in state

            operation = objFunction(**kwargs)
            
            timestamp = obj.save()
            operation["time"] = timestamp
            
            # Publish the value
            self.publish(obj.id, operation)
        
        # Execute the transation
        self.transaction(update_state_set, others = [], watches = [obj.id])
        
    def subscriber(self):
        return StateObjectSubscriber(self)
    



from pyramid.view import view_config
from pyramid.response import Response
import jarvis.server.redisconnect as redisconnect
import jarvis.server.utils as utils
from gevent.queue import Queue
import gevent
import json
import time
import os
import jarvis.server.state as state
import state_observer

STATE_PREFIX = "state-"

def state_id_key(id):
    return STATE_PREFIX + id

def decode_state(value):
    return json.loads(value)        

# Update a state
@view_config(route_name='state_update', request_method="POST")
def state_post(context, request):
    id = request.matchdict["id"]
    object_type = request.POST.get('type', 'string')
    operation_type = request.POST.get("op", "set")
    
    parameters = {}
    for k, v in request.POST.iteritems():
        if k not in ["op", "type"]:
            parameters[k] = v
            
    # Create redis client
    client = redisconnect.connect()

    # Create top level         
    st = state.State("user", client)

    obj = st.get_or_create(id, object_type)
        
    obj.op(operation_type, **parameters)
        
    resp = Response()
    resp.content_type = "text/plain"
    resp.text = u"OK"
    # return the old value of the key
    return resp
                
class StateObserverSessionSet(utils.GeventLockable):
    def __init__(self):
        super(StateObserverSessionSet, self).__init__()
        self.sessions = {}
        

    def get_observer(self, session, user, **kwargs):
        if session != None:
            session = "-".join([user, session])

        with self.lock():
            if session not in self.sessions:            
                queue = Queue()
                observer = self.factory(session, user, queue, **kwargs)
                if session != None:
                    # No session is required
                    self.sessions[session] = (queue, observer)
                observer.start()
            else:
                queue, observer = self.sessions[session]
                
            
        return queue, observer


def state_get(context, request, stateObserverSet, response, content_type, stream = True):
    session_id = request.matchdict["session_id"]
    request_dict = request.params
    max_messages = int(request_dict.get("count", -1))
    ids_string = request_dict.get("ids")

    if ids_string != None:
        ids = ids_string.split(",")
    else:
        ids = []

    user = "user"
    # Create response
    queue, observer = stateObserverSet.get_observer(session_id, user, max_messages = max_messages)
    
    for id in ids:
        observer.add_id(id)

    if stream:
        response.app_iter = queue
        response.content_type = content_type
    else:
        response.text == u"OK"
        response.content_type = "text/plain"
    return response

    
class JsonStateObserverSessionSet(StateObserverSessionSet):
    factory = state_observer.JsonStateObserver

jsonStateObserverSet = JsonStateObserverSessionSet()

@view_config(route_name='state_stream_json', request_method="GET")
def state_get_json(context, request):
    response = Response()
    return state_get(context, request, jsonStateObserverSet, response, 'application/json')
    

class EventSourceStateObserverSessionSet(StateObserverSessionSet):
    factory = state_observer.EventSourceStateObserver

eventSourceStateObserverSet = EventSourceStateObserverSessionSet()

@view_config(route_name='state_stream_eventsource', request_method="POST")
def state_get_event_source(context, request):
    event_stream = request.headers.get("accept") == "text/event-stream"

    response = Response()
    if event_stream:
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Cache-Control'] = 'no-cache'

    return state_get(context, request, eventSourceStateObserverSet, response, 'text/event-stream', stream = event_stream)





import sys
import __builtin__
import copy
import traceback

class RollbackImporter:
    def __init__(self):
        "Creates an instance and installs as the global importer"
        self.previousModules = sys.modules.copy()
        self.realImport = __builtin__.__import__
        __builtin__.__import__ = self._import
        self.newModules = []
        
    def _import(self, name, globals=None, locals=None, fromlist=[], level = -1):
        # Apply real import
        result = apply(self.realImport, (name, globals, locals, fromlist, level))

        if len(name) <= len(result.__name__):
           name = copy.copy(result.__name__)

        black_list = ["stashy", "unittest"] #, "sentry", "raven", "logging", "warnings"]
        
        for b in black_list:
            if b in name.lower():
                return result

        # Remember import in the right order
        if name not in self.previousModules:
            # Only remember once
            if name not in self.newModules:                            
                self.newModules += [name]
        return result

    def cleanup(self, display):
        for name in self.newModules:
            # Force reload when modname next imported
            try:
                reload(sys.modules[name])
            except TypeError:
                pass
            except Exception, e:
                display.errorprint("ERROR on ", traceback.format_exc(e), name)
#        self.newModules = []
                
    def uninstall(self):
        __builtin__.__import__ = self.realImport




import os
import os.path as op
import unittest
import shutil
import redis

import mox
import jarvis.commands as commands

import urllib
import time
import json
import gevent
import logging
log = logging.getLogger('eventsource.client')

this_dir = op.dirname(__file__)
data_dir = op.join(this_dir, "data")
tmp_dir = op.join(this_dir, "tmp")

def debug(*args):
    commands.debug(*args)

class Event(object):
    """
    Contains a received event to be processed
    """
    def __init__(self):
        self.type = None
        self.name = None
        self.data = None
        self.id = None

    def __repr__(self):
        return "Event<%s,%s,%s,%s>" % (str(self.type),  str(self.id), str(self.name), str((self.data or "").replace('\n','\\n')))


class EventSourceClient(object):
    def __init__(self):
        self.logging = False

    def debug(self, *args):
        if not self.logging:
            return
        log.debug(*args)

    def info(self, *args):
        if not self.logging:
            return
        log.info(*args)

    def parse(self, message):
        """
        Acts on message reception
        :param message: string of an incoming message

        parse all the fields and builds an Event object that is passed to the callback function
        """
        self.debug("handle_stream(...)")
        
        event = Event()

        for line in message.strip('\n').split('\n'):
            if len(line) == 0:
                yield event
                event = Event()
                continue

            (field, value) = line.split(":",1)
            event.type = field
            if field == 'event':
                event.name = value.lstrip()
            elif field == 'data':
                value = value.lstrip()
                if event.data is None:
                    event.data = value
                else:
                    event.data = "%s\n%s" % (event.data, value)
            elif field == 'id':
                event.id = value.lstrip()
            elif field == 'retry':
                try:
                    self.retry_timeout = int(value)
                    self.info( "timeout reset: %s" % (value,) )
                except ValueError:
                    pass
            elif field == '':
                self.info( "received comment: %s" % (value,) )
            else:
                raise Exception("Unknown field !")
        if event.type != None:
            yield event

    
class TestRoutes(unittest.TestCase):
    def setUp(self):
        # Refresh the temporary directory
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass
        os.makedirs(tmp_dir)
        
        try:
            os.makedirs(data_dir)
        except:
            pass
        
        self.mox = mox.Mox()
        
    def data_file_name(self, filename):
        return op.join(data_dir, filename)
    
    def get_tst_file_name(self, filename):
        return op.join(tmp_dir, filename)

    def tst_basic_send(self, msg):
        gevent.sleep(1.0)
        msg = {"op":"set", "content":str(msg)}
        msg = urllib.urlencode(msg)
        a = urllib.urlopen("http://localhost:9017/state/update/a/", msg).read()
        print "GOT", a
    
    def st_basic_json(self):
        client = redis.Redis("127.0.0.1", 6379)
        client.delete("state-user-a")
        oldtime = str(time.time())
        msg = {"op":"set", "content":oldtime}
        msg = urllib.urlencode(msg)

        a = urllib.urlopen("http://localhost:9017/state/update/a/", msg).read()

        newtime = time.time() + 1
        sublet = gevent.Greenlet.spawn(self.tst_basic_send, newtime)
        
        r = urllib.urlopen("http://localhost:9017/state/stream/json/?count=2&ids=a,b").read()
        r = r.split("\n")

        sublet.join()
        
        self.assertTrue(len(r) == 3)
        for i, b in enumerate(r):
            if b == '':
                continue
            b = json.loads(b)
            self.assertTrue("time" in b)
            del b["time"]

            data = [oldtime, newtime][i]
                
            ref = {u'user': u'user', u'op': u'set', u'id': u'a', u'parameters': {u'content': unicode(data)}}
            self.assertTrue(b == ref)
    


    def test_basic_eventsource(self):
        client = redis.Redis("127.0.0.1", 6379)
        client.delete("state-user-a")
        oldtime = str(time.time())
        msg = {"op":"set", "content":oldtime}
        msg = urllib.urlencode(msg)

        a = urllib.urlopen("http://localhost:9017/state/update/a/", msg).read()
        
        msg = {"ids":"a,b", "count":"2"}
        msg = urllib.urlencode(msg)

        newtime = time.time() + 1
        sublet = gevent.Greenlet.spawn(self.tst_basic_send, newtime)
        
        r = urllib.urlopen("http://localhost:9017/state/stream/eventsource/", msg).read()
        msgs = []

        esc =  EventSourceClient()
        for m in esc.parse(r):
            if m.type == "data":
                msgs += [m]

        r = msgs
                                
        self.assertTrue(len(r) == 2)
        
        for i, b in enumerate(r):
            if b == '':
                continue
            b = json.loads(b.data)
            self.assertTrue("time" in b)
            del b["time"]

            data = [oldtime, newtime][i]
                
            ref = {u'user': u'user', u'op': u'set', u'id': u'a', u'parameters': {u'content': unicode(data)}}
            
            self.assertTrue(b == ref)
    

        
    def tearDown(self):
        self.mox.UnsetStubs()
        
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass
        
def main():
    commands.add_watch_file(op.join(op.dirname(__file__), "../../views.py"))
    commands.add_watch_file(op.join(op.dirname(__file__), "../../state.py"))
    prefix = "test_basic_eventsource"
    suite = unittest.TestLoader().loadTestsFromTestCase(TestRoutes)
    suite = filter(lambda x : str(x).startswith(prefix), suite)
    suite = unittest.TestLoader().suiteClass(suite)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    commands.testunit_result(result)

if __name__ == "__main__":
    main()

    



import copy
import traceback
import os
import os.path
import errno

# Reserved settings names
RESERVED_NAMES = (
)

class _DummyContainer(object):
    pass

class Conf:
    """This object is responsible for reading configuration files that are written in python."""
    def __init__(self, pathes, defaults):
        # The list of places to look for configuration
        self.pathes = pathes
        # Default values for configuration
        self.defaults = defaults
        # Bare container for configuration
        self.container = _DummyContainer()
        # Then read configuration
        self.read()

    def valid_key(self, key):
        """Check if configuration key is valid"""
        if key.startswith("__") and key.endswith("__"):
            return False
        else:
            return True
        
    def set_key_value(self, key, value):
        """Set a new key in configuration"""
        if self.valid_key(key):
            setattr(self.container, key, value)        

    def read(self):
        """Read the configuration files after reading the defaults"""
        for key,value in self.defaults.iteritems():
            self.set_key_value(key, value)

        # Enumerate the pathes
        for p in self.pathes:
            try:
                data = {}
                s = execfile(p, data)
                # Set the keys, values from this configuration file
                for key, value in data.iteritems():
                    self.set_key_value(key, value)            
            except IOError, e:
                # Raise 
                if e.errno != errno.ENOENT:
                    raise

    def __getattr__(self, key):
        if self.valid_key(key):
            return getattr(self.container, key)
        else:
            raise AttributeError("%r object has no attribute %r" % (type(self).__name__, key))

    def __hasattr__(self, key):
        if self.valid_key(key):
            return hasattr(self.container, key)
        else:
            return False            
    
def load(name, pathes = None, defaults = {}):
    if pathes == None:
        pathes = []
        pathes += ["/etc/%s/conf.py" % name]
        pathes +=  [os.path.join(os.getenv("HOME"), "." + name + ".d", "conf.py")]
    else:
        pathes = copy.deepcopy(pathes)
    globals()[name] = Conf(pathes, defaults = defaults)
    



import os
import os.path as op
import unittest
import shutil

import mock 
import mox
import jarvis.utils.conf as conf
from jarvis.commands import *

this_dir = op.dirname(__file__)
data_dir = op.join(this_dir, "data")
tmp_dir = op.join(this_dir, "tmp")

class TestConf(unittest.TestCase):
    def setUp(self):
        # Refresh the temporary directory
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass
        os.makedirs(tmp_dir)

        try:
            os.makedirs(data_dir)
        except:
            pass
        
        self.mox = mox.Mox()
        
    def data_file_name(self, filename):
        return op.join(data_dir, filename)
    
    def get_test_file_name(self, filename):
        return op.join(tmp_dir, filename)

    def test_main(self):
        o = conf.Conf([self.data_file_name("serverconf.py")], {})
        self.assertTrue(o.a == "b")
        self.assertTrue(o.c == "bbbb")

    def test_load(self):
        conf.load("jarvis", pathes=[self.data_file_name("serverconf.py")])
        self.assertTrue(conf.jarvis.a ==  "b")
        self.assertTrue(conf.jarvis.c ==  "bbbb")

    def test_bad_load(self):
        with self.assertRaises(NameError):
            conf.load("jarvis", pathes= [self.data_file_name("serverconf_bad.py")])
        
    def tearDown(self):
        self.mox.UnsetStubs()
        
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass

        
def main():
    prefix = "test_"
    suite = unittest.TestLoader().loadTestsFromTestCase(TestConf)
    suite = filter(lambda x : str(x).startswith(prefix), suite)
    suite = unittest.TestLoader().suiteClass(suite)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    testunit_result(result)



import os.path
from Pymacs import lisp
import jinja2
import jinja2.meta


#def simplify_on_whitespace():
#    start, end = lisp.point(), lisp.mark(True)
#    words = lisp.buffer_substring(start, end).split()
#    words = filter(lambda x : x != "", words)
#    lisp.delete_region(start, end)
#    lisp.insert('K'.join(words))


# Special class to wrap commandfilename
class FunctionWrap(object):
    def __init__(self, commandfilename):
        self.commandfilename = commandfilename
        self.interaction = self.interaction_()

    def __call__(self, *args, **kwargs):
        f = open("/tmp/log2", "a")
        f.write(str(args) + "\n")
        f.write(str(kwargs) + "\n")
        f.close()
        exec(open(self.commandfilename).read(), {"arg":args, "kwargs":kwargs, "__file__":self.commandfilename})

    def interaction_(self):
        try:
            return open(self.commandfilename[:-3] + ".spec").read()
        except:
            return ""

class SnippetWrap(FunctionWrap):
    def __init__(self, commandfilename):
        e = jinja2.Environment()
        s = open(commandfilename).read()
        ast = e.parse(s)
        self.variables = list(jinja2.meta.find_undeclared_variables(ast))

        super(SnippetWrap, self).__init__(commandfilename)

    def interaction_(self):
        s = ""
        for v in self.variables:
            s += "s%s:\n" % v
        return s

    def __call__(self, *args):
        s = open(self.commandfilename).read()
        template = jinja2.Template(s)

        kwargs = {}
        for i, a in enumerate(args):
            kwargs[self.variables[i]] = a

        s = template.render(**kwargs)
        lisp.insert(s)

# Build the commands directory
dirname = os.path.dirname(os.path.abspath(__file__))

def get_user_dir(n):
    ret = os.path.join(os.getenv("HOME"), ".jarvis.d", n)
    # Create the directory
    if not os.path.exists(ret):
        os.path.makedirs(ret)
    return ret

def get_dirs(n):
    subdirnames = []
    for d in [os.path.join(os.getenv("HOME"), ".jarvis.d"), dirname]:
        subdirname = os.path.join(d, n)
        if os.path.exists(subdirname) and os.path.isdir(subdirname):
            subdirnames += [subdirname]
    return subdirnames

def get_command_file(command_name, create = False):
    for commanddirname in commanddirnames:
        # Enumerate the commands
        full_file_name = os.path.join(commanddirname, command_name + ".py")
        if os.path.exists(full_file_name):
            return full_file_name

    return None

commanddirnames = get_dirs("commands")

for commanddirname in commanddirnames:
    # Enumerate the commands
    for f in os.listdir(commanddirname):
        if f.endswith(".py"):
            commandname = f[:-3]
            commandfilename = os.path.join(commanddirname, f)
            if commandname not in globals():
                globals()[commandname] = FunctionWrap(commandfilename)

snippetdirnames = get_dirs("snippets")

for snippetdirname in snippetdirnames:
    # Enumerate the snippets
    for f in os.listdir(snippetdirname):
        if not f.endswith('~'):
            commandname = f
            commandfilename = os.path.join(snippetdirname, f)
            if commandname not in globals():
                globals()[commandname] = SnippetWrap(commandfilename)



import os
import os.path as op

from pyramid.config import Configurator
from gevent.pywsgi import WSGIServer

class Frontend(object):
    def start(self, start_wsgi_server=False, forever = False):
        """
        Start the frontend's greenlets.

        If *start_wsgi_server* is True, also create the frontend's main WSGI
        application and serve it with gevent's WSGI server.
        """
        # Start the frontend WSGI server if requested
        if start_wsgi_server:
            wsgi_app = self.create_wsgi_app({
                'pyramid.debug_authorization': 'true',
                'pyramid.debug_routematch': 'true',
                'pyramid.reload_templates': 'true',
                'pyramid.debug_notfound': 'true'
            })
            logfile = open("/tmp/jarvis.log", "w")

            self.wsgi_server = WSGIServer(("127.0.0.1", 9017), wsgi_app, log=logfile)
            if forever:
                self.wsgi_server.serve_forever()
            else:
                self.wsgi_server.start()
        else:
            self.wsgi_server = None

    def stop(self):
        if self.wsgi_server is not None:
            self.wsgi_server.stop()

    def create_wsgi_app(self, settings):
        # Create applincation
        config = Configurator(settings=settings)
        config.add_route('state_update', '/state/update/{id}/')
        config.add_route('state_stream_json', '/state/stream/json/{session_id}/')
        config.add_route('state_stream_eventsource', '/state/stream/eventsource/{session_id}/')
        static_path = op.join(op.dirname(__file__), "media")
        config.add_static_view(name='static', path=static_path)
        config.scan()

        return config.make_wsgi_app()



import json
import types

from redis.client import Redis

from jarvis.redis.base import RedisBase


class RedisHashDict(RedisBase):
    """
    Dict-like object on top of a Redis hash.
    """

    def __init__(self, client, name, load=None, dump=None):
        super(RedisHashDict, self).__init__(client, load=load, dump=dump)
        self.name = name

    def __getitem__(self, key):
        ret = self.raw_get(key)
        if ret is None:
            raise KeyError(key)
        return self.load_value(ret)

    def __setitem__(self, key, value):
        value = self.dump_value(value)
        self.client.hset(self.name, key, value)

    def raw_delete(self, key):
        return self.client.hdel(self.name, key)

    def __delitem__(self, key):
        if not self.raw_delete(key):
            raise KeyError(key)

    def __len__(self):
        return self.client.hlen(self.name)

    def __contains__(self, key):
        return self.client.hexists(self.name, key)

    def __iter__(self):
        return iter(self.keys())

    def keys(self):
        return self.client.hkeys(self.name)

    def values(self):
        return [self.load_value(v) for v in self.raw_values()]

    def raw_values(self):
        return self.client.hvals(self.name)

    def items(self):
        return [(k, self.load_value(v)) for k, v in self.raw_items().items()]

    def update(self, mapping):
        values = dict((k, self.dump_value(v)) for k, v in mapping.items())
        self.client.hmset(self.name, values)

    def setnx(self, key, value):
        """
        Set *key* with *value*, only if it doesn't exist.

        Return a boolean indicating if the key was modified.
        """
        return self.client.hsetnx(self.name, key, self.dump_value(value))

    def raw_items(self):
        """
        Retrieve raw key/value pairs (usefull in transactions).
        """
        return self.client.hgetall(self.name)

    def get(self, key, default=None):
        ret = self.raw_get(key)
        if ret is None:
            return default
        return self.load_value(ret)

    def raw_get(self, key):
        return self.client.hget(self.name, key)

    def getall(self, keys):
        """
        Get all values paired with the list of keys *keys*.
        """
        return [self.load_value(v) for v in self.raw_getall(keys)]

    def raw_getall(self, keys):
        return self.client.hmget(self.name, keys)


class RedisDict(RedisBase):
    """
    Dict-like object on top of Redis keys.

    The optional *ttl* parameter allows to specify a time-to-live for
    keys created with this object. If *ttl* is an integer then the Redis
    EXPIRE command is used; if it is a float the PEXPIRE command is used. In
    all cases *ttl* is specified in seconds.

    *prefix* may be a string defining the prefix of the keys. The default is
    no prefix, meaning the dict spans all the keys in the database.
    """

    def __init__(self, client, ttl=None, prefix='', load=None, dump=None):
        if not isinstance(ttl, (types.NoneType, int, float)):
            raise TypeError('ttl must be None, an int or a float')
        super(RedisDict, self).__init__(client, load=load, dump=dump)
        self._ttl = ttl
        self.prefix = prefix

    def __getitem__(self, key):
        key = self.raw_key(key)
        ret = self.client.get(key)
        if ret is None:
            raise KeyError(key)
        return self.load_value(ret)

    def __setitem__(self, key, value):
        key = self.raw_key(key)
        value = self.dump_value(value)
        if self._ttl is None:
            self.client.set(key, value)
        else:
            if isinstance(self._ttl, int):
                duration = self._ttl
                func = self.client.setex
            elif isinstance(self._ttl, float):
                duration = int(self._ttl) * 1000
                func = self.client.psetex
            if isinstance(self.client, Redis):
                func(key, value, duration)
            else:
                func(key, duration, value)

    def __delitem__(self, key):
        key = self.raw_key(key)
        self.client.delete(key)

    def setnx(self, key, value):
        """
        Set *key* with value, only if *key* does not exist.

        Returns a boolean indicating if the key was created.

        .. warning:: this method does **NOT** set an expiration time on the
                     created key.
        """
        key = self.raw_key(key)
        value = self.dump_value(value)
        return self.client.setnx(key, value)

    def __contains__(self, key):
        key = self.raw_key(key)
        return self.client.exists(key)

    def get(self, key, default=None):
        key = self.raw_key(key)
        value = self.client.get(key)
        if value is None:
            return default
        return self.load_value(value)

    def raw_key(self, key):
        """
        Get the raw key name of *key*, as it is stored in Redis.
        """
        return self.prefix + key

    def keys(self):
        """
        Return the keys in this dict with their prefix removed.
        """
        return [k[len(self.prefix):] for k in self.raw_keys()]

    def raw_keys(self):
        """
        Return the keys in this dict, with their prefix.
        """
        return self.client.keys(self.prefix + '*')

    def values(self):
        """
        Return the values in this dict.
        """
        keys = self.raw_keys()
        if not keys:
            return []
        return [self.load_value(v)
                for v in self.client.mget(*keys) if v is not None]

    def items(self):
        """
        Return ``(key, value)`` pairs in this dict.
        """
        keys = self.raw_keys()
        if not keys:
            return []
        values = self.client.mget(*keys)
        return [(k[len(self.prefix):], self.load_value(v))
                for k, v in zip(keys, values) if v is not None]

    def expire(self, key, duration=None):
        """
        Refresh or set the expiration timeout of *key*.

        The default :attr:`ttl` duration is used, or *duration* may be
        specified to override it (in seconds).
        """
        key = self.raw_key(key)
        if duration is None:
            if self._ttl is None:
                raise ValueError("the dict doesn't have a default "
                        "expiration set, please pass a value to the "
                        "duration parameter")
            duration = self._ttl
        if isinstance(duration, int):
            self.client.expire(key, duration)
        elif isinstance(duration, float):
            self.client.pexpire(key, int(duration * 1000))

    def ttl(self, key):
        """
        Get the remaining time to live of *key*, in seconds.

        The method tries to get the most precise answer, first trying PTTL,
        then TTL if the client doesn't support it. Returns None if the key
        doesn't have an expiration set.
        """
        try:
            ttl = self.raw_pttl(key)
            if ttl is not None:
                return ttl / 1000.0
        except AttributeError:
            ttl = self.raw_ttl(key)
            if ttl is not None:
                return float(ttl)

    def raw_ttl(self, key):
        """
        Raw call to :meth:`redis.client.Redis.ttl`.
        """
        key = self.raw_key(key)
        return self.client.ttl(key)

    def raw_pttl(self, key):
        """
        Raw call to :meth:`redis.client.Redis.pttl`.
        """
        key = self.raw_key(key)
        return self.client.pttl(key)


class RedisHashJsonDict(RedisHashDict):

    def __init__(self, client, name):
        super(RedisHashJsonDict, self).__init__(client, name, load=json.loads,
                dump=json.dumps)


class RedisJsonDict(RedisDict):
    def __init__(self, client, ttl=None, prefix=''):
        super(RedisJsonDict, self).__init__(client, ttl=ttl, prefix=prefix,
                load=json.loads, dump=json.dumps)



from __future__ import absolute_import

import redis

import jarvis.utils.conf as conf

connection_pool = redis.ConnectionPool()

def connect(host=None, port=None, password=None, db=None):
    """
    Create a :class:`redis.Redis` connection object, using Dragon's default
    configuration.
    """
    host = host if host is not None else conf.jarvis.REDIS_HOST if hasattr(conf.jarvis, "REDIS_HOST") else "localhost"
    port = port if port is not None else conf.jarvis.REDIS_PORT if hasattr(conf.jarvis, "REDIS_PORT") else 6379
    password = password if password is not None else conf.jarvis.REDIS_PASSWORD if hasattr(conf.jarvis, "REDIS_PASSWORD") else None
    db = db if db is not None else conf.jarvis.REDIS_DB
    return redis.Redis(host=host, port=port, password=password, db=db, connection_pool=connection_pool)




class Lisp(object):
    def __init__(self):
        pass

    def buffer_file_name(self):
        return raw_input("input filename:")

    def completing_read(self, message, options):
        ret = None
        while ret not in options:
            ret = raw_input(message + " Available options are: " + ",".join(options) + "  \n")
            if ret in options:
                return ret
            else:
                print "Invalid option"
                    
    def find_file(self, filename):
        print "File was created at\n %s" % filename

    def j_test_filename_function_set(self, *args, **kwargs):
        pass

    def insert(self, s):
        print s



import contextlib

import redis.client


class RedisBase(object):
    """
    Base class for redis stores. *load* and *dump* may be callables used to
    transform values from strings to python objects, and from python objects to
    strings (e.g. :func:`json.loads` and :func:`json.dumps`).
    """

    def __init__(self, client, load=None, dump=None):
        self.client = client
        self.load = load
        self.dump = dump

    def load_value(self, value):
        if self.load is None:
            return value
        else:
            return self.load(value)

    def dump_value(self, value):
        if self.dump is None:
            return value
        else:
            return self.dump(value)

    def use_pipe(self, pipe):
        """
        A context manager to temporary run the commands of this object through
        *pipe*, a :class:`redis.client.Pipeline` object.
        """
        return PipelineContextManager(self, pipe)

    def _check_multi_return(self, ret):
        """
        Search exceptions in the MULTI return value *ret*, and raise the first
        found.
        """
        for value in ret:
            if isinstance(value, Exception):
                raise value

    def transaction(self, func, others=None, watches=()):
        """
        Run *func* in a transaction, possibly with other :class:`RedisBase`
        objects.
        """
        # Verify this object is not already used in a transaction
        if isinstance(self.client, redis.client.Pipeline):
            raise Exception("nested transactions are not allowed")

        # Get watches names
        if not isinstance(watches, (list, tuple)):
            watches = [watches]
        watches_names = [w.name if isinstance(w, RedisBase) else w
                for w in watches]

        # Check that all the objects use the same redis client
        objs = [self]
        if others is not None:
            if not isinstance(others, (list, tuple)):
                others = [others]
            objs.extend(others)


            clients = [o.client for o in objs]
            if not clients.count(clients[0]) == len(objs):
                for i, obj in enumerate(objs):
                    print "OBJ, CLIENT", obj, clients[i]
                raise ValueError("all objects must use the same redis client: " + str(len(objs)) + " "  + str(clients.count(clients[0])))
            client = clients[0]
        else:
            client = self.client

        # Run func in a transaction
        def transaction(pipe):
            with contextlib.nested(*(o.use_pipe(pipe) for o in objs)):
                func(pipe)

        ret = client.transaction(transaction, *watches_names)
        self._check_multi_return(ret)
        return ret


class PipelineContextManager(object):
    """
    Used by :meth:`RedisBase.use_pipe` to temporary substitute a redis client
    with a :class:`redis.client.Pipeline` object.
    """

    def __init__(self, redis_obj, pipe):
        self.redis_obj = redis_obj
        self.pipe = pipe

    def __enter__(self):
        self.old_client = self.redis_obj.client
        self.redis_obj.client = self.pipe

    def __exit__(self, *exc_info):
        self.redis_obj.client = self.old_client



from gevent import monkey;
monkey.patch_all()
import gevent

import os
import os.path as op
import unittest
import shutil

import mox
import jarvis.server.broker as broker
import jarvis.utils.conf as conf


this_dir = op.dirname(__file__)
data_dir = op.join(this_dir, "data")
tmp_dir = op.join(this_dir, "tmp")

class TestJarvisServerBroker(unittest.TestCase):
    def setUp(self):
        conf.load("jarvis", defaults = {"REDIS_DB":10})

        # Refresh the temporary directory
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass
        os.makedirs(tmp_dir)

        try:
            os.makedirs(data_dir)
        except:
            pass
        
        self.mox = mox.Mox()
        
    def data_file_name(self, filename):
        return op.join(data_dir, filename)
    
    def get_tst_file_name(self, filename):
        return op.join(tmp_dir, filename)

    def tst_basic_subscribe(self, broker):
        sub = broker.subscriber()
        sub.subscribe("test_channel")
        for s in sub:
            return s
        
        
    def tst_basic_publish(self, broker):
        pub = broker.publisher()
        pub.publish("test_channel", {"a":"b"})
        return "OK"
    
    def test_basic(self):
        # TODO
        b = broker.Broker()
        
        sublet = gevent.Greenlet.spawn(self.tst_basic_subscribe, b)
        publet = gevent.Greenlet.spawn(self.tst_basic_publish, b)

        jobs = [sublet, publet]
        gevent.joinall(jobs, timeout=2)

        ret = [job.value for job in jobs]
        self.assertEquals(ret,  [('test_channel', {u'a': u'b'}), 'OK'])
        
    
    def tearDown(self):
        self.mox.UnsetStubs()
        
        try:
            shutil.rmtree(tmp_dir, True)
        except:
            pass

        
def main():
    prefix = "test_"
    suite = unittest.TestLoader().loadTestsFromTestCase(TestJarvisServerBroker)
    suite = filter(lambda x : str(x).startswith(prefix), suite)
    suite = unittest.TestLoader().suiteClass(suite)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
#    testunit_result(result)

if __name__ == "__main__":
    main()
    



import numpy
import wave

class WaveReader():
    def __init__(self, filename):
        self.filename = filename
        self.reader = wave.open(filename)

    def read(self, length):
        out = self.reader.readframes(length)
        return numpy.fromstring(out, dtype=numpy.dtype('<i2'))

class WaveSink(object):
    # max is used to renormalize audio from float stream to short int stream
    def __init__(self, input, outputFileName, duration, skip = 0.0, framerate = 44100, channels = 2):
        self.framerate = framerate
        self.channels = channels
        self.input = input
        self.duration = duration
        self.skip = skip
        self.bufferSamples = int(self.channels * self.framerate * self.duration)

        output = wave.open(outputFileName, "w")
        output.setnchannels(channels)
        output.setsampwidth(2)
        output.setframerate(framerate)

        self.output = output

    def sizeToTime(self, size):
        return float(size) / float(self.framerate * self.channels * 2)

    def timeToSize(self, time):
        return int(time * self.framerate * self.channels * 2)

    def run(self):
        previousTotalRead = 0
        totalRead = 0

        BLOCK_SIZE = int(min(1.0, self.duration / 3.0) * self.framerate)

        skipSize = self.timeToSize(self.skip)
        totalSize = self.timeToSize(self.skip + self.duration)        
        
        while(True):            
            out = self.input.read(BLOCK_SIZE)

            if out == None:
                break
            
            ar = numpy.array(out, dtype=numpy.dtype('<i2'))
            outStr = ar.tostring()
            
            if len(outStr) == 0:
                break

            previousTotalRead = totalRead
            totalRead += len(outStr)
            
            if totalRead <= skipSize:
                continue
            if previousTotalRead > totalSize:
                break
            if previousTotalRead < skipSize and totalRead > skipSize:
                outStr = outStr[skipSize - previousTotalRead:]                
            if totalRead > totalSize:
                outStr = outStr[: - (totalRead - totalSize)]
                
            self.output.writeframes(outStr)
            
        self.output.close()


    def close(self):
        try:
            self.output.close()
        finally:
            self.input.close()            



import json

from jarvis.redis.base import RedisBase

class RedisPublisher(RedisBase):
    """
    Redis-based publisher.
    """

    def __init__(self, client, dump=None):
        super(RedisPublisher, self).__init__(client, dump=dump)

    def publish(self, channel, message):
        """
        Wrapper for :meth:`redis.Redis.publish`.
        """
        message = self.dump_value(message)
        self.client.publish(channel, message)


class RedisSubscriber(RedisBase):
    """
    Redis-based subscriber.
    """

    def __init__(self, client, load=None):
        super(RedisSubscriber, self).__init__(client, load=load)
        self.pubsub = client.pubsub()

    def psubscribe(self, channels):
        """
        Subscribe to all channels matching any pattern in *patterns*.
        """
        self.pubsub.psubscribe(channels)
        self.pubsub.parse_response()

    def punsubscribe(self, channels=[]):
        """
        Unsubscribe from any channel matching any pattern in *patterns*.  If
        empty, unsubscribe from all channels.
        """
        self.pubsub.punsubscribe(channels)
        self.pubsub.parse_response()

    def subscribe(self, channels):
        """
        Subscribe to *channels*, waiting for messages to be published.
        """
        self.pubsub.subscribe(channels)
        self.pubsub.parse_response()

    def unsubscribe(self, channels=[]):
        """
        Unsubscribe from *channels*. If empty, unsubscribe from all channels.
        """
        self.pubsub.unsubscribe(channels)
        self.pubsub.parse_response()

    def recv(self):
        """
        Get the next message.

        Returns a ``(channel, message)`` tuple. This method will block if there
        is nothing in the subscribed channels.
        """
        return next(iter(self))

    def __iter__(self):
        """
        Yield ``(channel, message)`` tuples as messages arrive on subscribed
        channels.
        """
        for item in self.pubsub.listen():
            if item["type"] == "message":
                yield item["channel"], self.load_value(item["data"])


class RedisJsonPublisher(RedisPublisher):
    """
    Convenience class for publishing JSON data.
    """

    def __init__(self, client):
        super(RedisJsonPublisher, self).__init__(client, dump=json.dumps)


class RedisJsonSubscriber(RedisSubscriber):
    """
    Convenience class for subscribing to JSON channels.
    """

    def __init__(self, client):
        super(RedisJsonSubscriber, self).__init__(client, load=json.loads)




import jarvis.utils.conf as conf
import urllib

conf.load("jarvis", defaults = {"SERVER_ADDRESS":"127.0.0.1", "SERVER_PORT":9107})

class Client(object):
    def __init__(self):
        self.address = conf.jarvis.SERVER_ADDRESS
        self.port = conf.jarvis.SERVER_PORT
        

    def update(self, id, op, content):
        body = urllib.urlencode({"op":op, "content":content})
        print "CONF JARVIS", dir(conf.jarvis)
        base_url = "http://%s:%s/state/update/" % (conf.jarvis.SERVER_ADDRESS, conf.jarvis.SERVER_PORT)
        response = urllib.urlopen(base_url + id + "/", body).read()
        print "response=", response
        if response != "OK":
            raise Exception(response)



import os
import os.path
import sys
import time
import traceback
from PyQt4 import QtCore
import jarvis
import rollbackimporter
import imp
import errno
import tracer

class Display():
    def __init__(self):
        pass

    def init(self):
        pass

    def destroy(self):
        pass

    def start(self):
        pass

    def finish(self):
        pass

    def debugprint(self, *args):
        info = " ".join(map(lambda x: str(x), args)) + "\n"
        print "INFO", info

    def errorprint(self, *args):
        info = " ".join(map(lambda x: str(x), args)) + "\n"
        print "INFO", info


class MainLoop(QtCore.QThread):
    def __init__(self, filename_function, display = None):
        QtCore.QThread.__init__(self)

        self.filename_function = filename_function
        self.finished = False
        self.display = display
        self.module = None

        self.rollbackImporter = rollbackimporter.RollbackImporter()
        self.watchfiles = {}
        # This flag says if last test was runned or not yet finished
        self.run_finished = True
        # Used to debug vars
        self.tracer = None
        self.file_dates = {}
        self.trace_file_white_list = {}
        # Set this flag to True to ask for a new run (True for the first run !)
        self.force_run = True

    def add_watch_file(self, filename):
        self.watchfiles[filename] = True

    def setdisplay(self, display):
        self.display = display

    def checkreloadfile(self, filename):
        modified = False
        try:
            filedate = os.path.getmtime(filename)
            self.watchfiles[filename] = True
            checkedfile = filename
        except OSError:
            return False, None

        old_file_date = self.file_dates.get(filename)

        if old_file_date != None and old_file_date != filedate:
            modified = True

        self.file_dates[filename] = filedate

        return modified, checkedfile

    def checkreloadmodule(self, module):
        checkedfiles = []
        if not hasattr(module, "__file__"):
            return False, []

        filename = module.__file__

        if filename == None:
            return False, []

        if filename.endswith(".pyc"):
            filename = filename[:-1]

        modified = False
        for suffix in [""]:
            localModified, checkedfile = self.checkreloadfile(filename + suffix)
            modified = modified or localModified
            if checkedfile != None:
                checkedfiles += [checkedfile]

        return modified, checkedfiles

    def get_test_filename(self):
        parts = self.filename_function.split(":")
        return parts[0]

    def get_test_fun_name(self):
        parts = self.filename_function.split(":")
        filename = parts[0]
        if len(parts) > 1:
            function_name = parts[1]
        else:
            function_name = "main"
        return function_name

    def loadMainModule(self):
        entry_point = self.get_test_filename()
        try:
            self.module = imp.load_source("__mainjarvismodule__", entry_point)
        except IOError, e:
            if e.errno == errno.ENOENT:
                raise Exception("The entry_point %s does not exist.", entry_point)

    def modulechanged(self):
        # Check that main module has not changed
        test_filename_function_path = os.path.join(jarvis.get_home(), jarvis.TEST_FILENAME_FUNCTION)

        test_filename_function = open(test_filename_function_path).read()
        if test_filename_function != self.filename_function:
            self.filename_function = test_filename_function
            return True
        return False

    def singlePrepare(self):
        modified = False

        # Check if any file was modified since last run
        modules = sys.modules.values()

        fullcheckedfiles = {}

        for m in modules:
            if m != None:
                try:
                    checkreload, checkedfiles = self.checkreloadmodule(m)
                    for filename in checkedfiles:
                        fullcheckedfiles[filename] = True
                    modified = modified or checkreload
                    if modified:
                        break
                except Exception, e:
                    print traceback.format_exc(e)

        for filename in self.watchfiles:
            if filename not in fullcheckedfiles:
                checkreload, checkedfile = self.checkreloadfile(filename)
                if checkreload:
                    modified = True
                    break


        return modified

    def start_run(self):
        self.run_finished = False

    def finish_run(self):
        self.run_finished = True

    def runcommand(self):
        if self.module != None:
            try:
                self.last_run = time.time()
                fun = getattr(self.module, self.get_test_fun_name())
                self.tracer = tracer.Tracer(self.trace_file_white_list)
                self.tracer.install()
                fun()
                self.tracer.uninstall()
                # Reset trace so it is recomputed
                self.trace_file_date = None
            except Exception, e:
                print traceback.format_exc(e)
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                self.display.errorprint("%s:%s\n" % (fname, exc_tb.tb_lineno) + traceback.format_exc(e) + "\n")
            finally:
                # We are sure we have run code that was up to date = self.new_check_timestamp
                self.finish_run()


    def inspect_vars(self, file, line):
        if self.tracer == None:
            return None
        else:
            return self.tracer.inspect(file, line)

    def singleRun(self):
        if self.display != None:
            self.display.runcommand(self.runcommand)


    def runloop(self):
        if not self.run_finished:
            return

        modified_force = self.force_run
        # Reset force_run
        self.force_run = False
        modified = modified_force

        module_changed = self.modulechanged()

        if module_changed:
            # The tested module changed, so we reset the set of files to be traced
            self.trace_file_white_list = {}
            self.display.reset()

        modified = modified or module_changed

        modified_single = self.singlePrepare()

        modified = modified or modified_single

        if not modified:
            self.finish_run()
            return

        self.loadMainModule()
        self.display.reset()

        self.start_run()

        self.display.start()

        if self.rollbackImporter:
            self.rollbackImporter.cleanup(self.display)

        self.singleRun()


    def trace_query_check_write(self, query_string, content):
        filename = jarvis.get_filename(jarvis.INSPECT_VAR_QUERY_RESPONSE)
        f = open(filename, "w")
        f.write(query_string + "\n")
        f.write(content + "\n")
        f.close()

    def trace_query_check(self):
        if self.tracer == None:
            return

        filename = jarvis.get_filename(jarvis.INSPECT_VAR_QUERY)

        # Check date of trace file, not to redo work for nothing
        if not os.path.exists(filename):
            return

        filedate = os.path.getmtime(filename)
        if hasattr(self, "trace_file_date") and filedate == self.trace_file_date:
            return

        try:
            query_string = open(filename).read()
            query = query_string.split()
            line = query[0]
            file = query[1]
        except:
            return None

        if not self.tracer.istracing(file):
            self.trace_file_white_list[file] = True
            # Force a new run !!
            self.force_run = True
            return

        ret = self.tracer.inspect(file, int(line))
        if ret == None:
            self.trace_query_check_write(query_string, "Function was not executed")
            return

        # The check was successful, so we can set the date
        self.trace_file_date = filedate

        new_ret = []
        for time,d in ret:
            new_ret += [[time, {}]]
            for k,v in d.iteritems():
                new_ret[-1][1][k] = [v[0], str(v[1])]
        ret = new_ret

        self.trace_query_check_write(query_string, str(ret))
#        self.trace_query.inspect_all()

    def run_(self):
        while(not self.finished):

            try:
                 self.runloop()
            except:
                self.finish_run()
                print traceback.format_exc()
                if self.display != None:
                    self.display.start()
                    self.display.errorprint(traceback.format_exc())
                    self.display.finish()

            try:
                self.trace_query_check()
            except:
                print traceback.format_exc()

            time.sleep(0.1)

    def run(self):
        # Write the current module name to disk
        testmodulepath = os.path.join(jarvis.get_home(), jarvis.TEST_FILENAME_FUNCTION)
        if self.filename_function != None:
            f = open(testmodulepath, "w")
            f.write(self.filename_function)
            f.close()

        try:
            self.run_()
        except KeyboardInterrupt:
            pass
        except Exception, e:
            print traceback.format_exc(e)



import os.path
import jinja2
import imp
import time
import jarvis.emacs as emacs
import os
from Pymacs import lisp

dirname = os.path.dirname(os.path.abspath(__file__))
snippetdirnames = []

snippetdirnames = emacs.get_dirs("snippets")

def get_snippet_dir_names():
    return snippetdirnames

def load_snippet(commandname, **kwargs):
    for snippetdirname in snippetdirnames:
        for suffix in ["", ".py"]:
            commandfilename = os.path.join(snippetdirname, commandname + suffix)

            if os.path.exists(commandfilename):
                break

    s = open(commandfilename).read()

    template = jinja2.Template(s)

    s = template.render(**kwargs)

    return s

def command_spec_run(spec):
    spec = map(lambda x:x.strip(), spec.split("\n"))
    spec = filter(lambda x:len(x) != 0, spec)
    args = []
    kwargs = {}
    for s in spec:
        type = s[0]
        name = s[1:-1]
        b = raw_input(name + ":")
        args += [b]
        kwargs[name] = b
    return args, kwargs

def command_run():
    import sys
    if len(sys.argv) < 2:
        print "USAGE: jarvis_run COMMAND_NAME [DESTINATION_FILE]"
        return
    command_name = sys.argv[1]
    if not hasattr(emacs, command_name):
        print "Unknown command %s" % command_name
        return
    command = getattr(emacs, command_name)
    if isinstance(command, emacs.SnippetWrap):
        if len(sys.argv) < 3:
            print "USAGE: jarvis_run COMMAND_NAME DESTINATION_FILE"
            return
        spec = command.interaction
        args, kwargs = command_spec_run(spec)

        s = load_snippet(command_name, **kwargs)
        dest_file = open(sys.argv[2], "w")
        dest_file.write(s)
        dest_file.close()
    elif isinstance(command, emacs.FunctionWrap):
        import Pymacs
        import pseudolisp
        savelisp = Pymacs.lisp
        Pymacs.lisp = pseudolisp.Lisp()
        spec = command.interaction
        args, kwarss = command_spec_run(spec)
        command(*args)
        Pymacs.lisp = savelisp

# Find the "tests" directory
def find_tests_path(filename):
    new_directory = filename
    test_path = None
    while(True):
        old_directory = new_directory
        test_path_try = os.path.join(new_directory, "tests")
        if os.path.exists(test_path_try) and os.path.isdir(test_path_try):
            test_path = test_path_try
            break

        new_directory = os.path.abspath(os.path.join(new_directory, ".."))

        if new_directory == old_directory:
            break
    return test_path


def find_module_name(filename):
    basename = os.path.basename(filename)
    if basename == "__init__.py":
        module_name = []
    else:
        module_name = [basename[:-3]]

    current_dir = os.path.dirname(filename)

    while(True):
        old_current_dir = current_dir
        initfile = os.path.join(current_dir, "__init__.py")
        if os.path.exists(initfile):
            module_name = [os.path.basename(current_dir)] + module_name
        else:
            break

        current_dir = os.path.abspath(os.path.join(current_dir, ".."))

        if current_dir == old_current_dir:
            break
    return ".".join(module_name)

def load_module_by_filename(filename):
    return imp.load_source("entrypoint" + str(time.time()).replace(".", "_"), filename)

def create_init_files(filename):
    current_dir = os.path.dirname(filename)

    while(True):
        old_current_dir = current_dir
        initfile = os.path.join(current_dir, "__init__.py")
        if os.path.exists(initfile):
            break
        else:
            f = open(initfile, "w")
            f.close()

        current_dir = os.path.abspath(os.path.join(current_dir, ".."))

        if current_dir == old_current_dir:
            break

def module_load_source(filename):
    import imp
    name = "entrypoint" + str(time.time()).replace(".", "_")

    pyc_filename = filename + "c"
    if os.path.exists(pyc_filename) and filename.endswith(".py"):
        os.remove(pyc_filename)

    mod = imp.load_source(name, filename)
    return mod

def find_class_names(filename):
    import inspect

    class_names = []
    # Load the module
    f = module_load_source(filename)

    # Enumerate the classes in the module
    for k in dir(f):
        # Get the task class
        cl = getattr(f, k)
        if inspect.isclass(cl):
            class_names  += [cl.__name__]

    return class_names

def cursor_line_number():
    point = lisp.point()
    line = lisp.count_lines(1, point + 1)
    return line


    new_var_info = ""
    infos = eval(var_info.split("\n")[1])
    for time, info in infos:
        new_var_info  += "Executed at %ss\n" % time

        context = []
        for k,v in info.iteritems():
            context += [(k,v)]

        context.sort(key = lambda x : - x[1][0])

        for k, info in context:
            new_var_info += "  %s => %s\n" % (k, info[1])

def inspect_format(var_info):
    new_var_info = ""
    infos = eval(var_info.split("\n")[1])
    for time, info in infos:
        new_var_info  += "Executed at %ss\n" % time

        context = []
        for k,v in info.iteritems():
            context += [(k,v)]

        context.sort(key = lambda x : - x[1][0])

        for k, info in context:
            new_var_info += "  %s => %s\n" % (k, info[1])

    var_info = new_var_info

    return var_info

global_vars = {}

def get_command_global_var(key, default = None):
    global global_vars

    return global_vars.get(key, default)

def set_command_global_var(key, value):
    global global_vars

    global_vars[key] = value




import jarvis.utils.conf as conf
import os.path
import jarvis.client as client

conf.load("jarvis")

class Agent(object):
    def __init__(self):
        # Build the name parts
        parts = ["agents", self.__class__.__name__.lower()]
        # Build the name
        name = "_".join(parts)
        # Build the config file name
        subpath = os.path.join(*parts) + ".py"
        # Assemble the config pathes
        basepathes = ["/etc/jarvis", os.path.join(os.getenv("HOME"), ".jarvis.d", "conf")]
        pathes = []
        for p in basepathes:            
            pathes += [os.path.join(p, subpath)]

        # Load the conf
        conf.load(name, pathes = pathes)

        # Retrieve the object containing the conf
        self.conf = getattr(conf, name)

        self.client = client.Client()
        
        
        




from gevent import monkey;
monkey.patch_all()

import logging
import logging.config
import jarvis.utils.conf as conf
import frontend

#from stupeflix.conf import settings as dragon_settings
#import dragon.conf
#import dragon_front.conf


import code, traceback, signal

def debug(sig, frame):
    """Interrupt running process, and provide a python prompt for
    interactive debugging."""
    d={'_frame':frame}         # Allow access to frame object.
    d.update(frame.f_globals)  # Unless shadowed by global
    d.update(frame.f_locals)

    i = code.InteractiveConsole(d)
    message  = "Signal recieved : entering python shell.\nTraceback:\n"
    message += ''.join(traceback.format_stack(frame))
    i.interact(message)

def listen():
    signal.signal(signal.SIGUSR1, debug)  # Register handler


def main(**settings):
    """
    This function returns a Pyramid WSGI application.
    """
    listen()

    # Load dragon configuration
    conf.load("jarvis")

    # Setup logging
    #logging.config.dictConfig(jarvis.LOGGING)

    # Start the frontend greenlets and create the WSGI application
    _frontend = frontend.Frontend()
    _frontend.start(start_wsgi_server = True, forever = True)



#!/usr/bin/env python
#osgviewerQt4.py

__author__  = ["Rene Molenaar"]
__url__     = ("http://code.google.com/p/osgswig/")
__version__ = "1.0.0"
__doc__     = """ This example shows to use osgViewer.Viewer within PyQt4 ____Rene Molenaar 2008
                  the basics functionality is present, but there are many 'missing features'
                  for example, mouse modifiers are not adapted, mouse wheel is not adapted and
                  non-ascii keyboard input is not adapted.
              """

# general Python
import sys,os

import OpenGL.GL as GL

# import qtWidgets stuff
import PyQt4

# import OpenSceneGraph wrapper
import osg, osgDB, osgGA, osgViewer

from PyQt4 import QtOpenGL
from PyQt4 import Qt, QtGui, QtCore
import time
import osgUtil
import jarvis

UPDATE_MASK=3
VISIBLE_CULL_MASK=1

mouseButtonDictionary = {
    QtCore.Qt.LeftButton: 1,
    QtCore.Qt.MidButton: 2,
    QtCore.Qt.RightButton: 3,
    QtCore.Qt.NoButton: 0,
    }

class PyQtOSGWidget(QtOpenGL.QGLWidget):
    def __init__(self, parent = 0, name = '' ,flags = 0):
        """constructor """
        QtOpenGL.QGLWidget.__init__(self, parent)
        self.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.timer = Qt.QTimer()
        self.timer.setInterval(40)
        self.camera = None
        self.startTime = time.time()
        self.loopTime = 10.0
        self.mousePressed = False
        self.current_mouse_time = 0
        self.audio = None

    def initializeGL (self):
        """initializeGL the context and create the osgViewer, also set manipulator and event handler """
        self.gw = self.createContext()
        self.viewer = self.createViewer()
        #init the default eventhandler
#        self.viewer.setCameraManipulator(osgGA.TrackballManipulator())
        self.viewer.addEventHandler(osgViewer.StatsHandler())
        self.viewer.addEventHandler(osgViewer.HelpHandler())
        self.viewer.getUpdateVisitor().setTraversalMask(UPDATE_MASK)
        
        QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout ()"), self.updateGL)
        self.timer.start(40)

    def setLoopTime(self, loopTime):
        self.loopTime = loopTime

    def embedInContext (self):
        """create a osg.GraphicsWindow for a Qt.QWidget window"""
        gw = osgViewer.GraphicsWindowEmbedded(0,0,self.width(),self.height())
        return gw

    def createContext (self):
        """create a opengl context (currently WindowData classes are not wrapped so we can not inherrit the windowdata) """
        ds = osg.DisplaySettings_instance()
        if False:
            traits = osg.GraphicsContext.Traits()
            print traits
            traits.readDISPLAY()
            if (traits.displayNum<0): traits.displayNum = 0

            traits.windowName = "osgViewerPyQt"
            traits.screenNum = 0
            traits.x = self.x()
            traits.y = self.y()
            traits.width = self.width()
            traits.height = self.height()
            traits.alpha = 8 #ds.getMinimumNumAlphaBits()
            traits.stencil = 8 #ds.getMinimumNumStencilBits()
            traits.windowDecoration = False
            traits.doubleBuffer = True
            traits.sampleBuffers = 4 #ds.getMultiSamples()
            traits.samples = 4 #ds.getNumMultiSamples()
        gw = osgViewer.GraphicsWindowEmbedded()
        return gw

    def createViewer(self):
        """create a osgViewer.Viewer and set the viewport, camera and previously created graphical context """
        viewer = osgViewer.Viewer()
        self.resetCamera(viewer)
        return viewer

    def resetCamera(self, viewer):

        camera = viewer.getCamera()
        camera.setComputeNearFarMode(False)
#        camera = osg.Camera()
        camera.setViewport(osg.Viewport(0,0, self.width(), self.height()))
#        camera.setReferenceFrame(osg.Transform.ABSOLUTE_RF)
        CAMERA_ANGLE = 45.0
        CAMERA_Z_TRANSLATE = 2.4142135623730949 #1.0 / math.tan(math.radians(CAMERA_ANGLE / 2.0))
        cameraPosition = [0.0, 0.0, CAMERA_Z_TRANSLATE]

        camera.setProjectionMatrixAsPerspective(CAMERA_ANGLE,float(self.width())/float(self.height()), 0.1, 100.0)

        eye = osg.Vec3d(cameraPosition[0], cameraPosition[1], cameraPosition[2])
        center = osg.Vec3d(0,0,0)
        up = osg.Vec3d(0,1,0)
        camera.setViewMatrixAsLookAt(eye, center, up)

        camera.getOrCreateStateSet().setAttributeAndModes(osg.BlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA))
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_TEST, False)
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_WRITEMASK, False)
        camera.getOrCreateStateSet().setMode(GL.GL_LIGHTING, False)
        material = osg.Material()
        color = osg.Vec4(1.0,1.0,1.0,1.0)
        material.setDiffuse(osg.Material.FRONT_AND_BACK, color)
        material.setAmbient(osg.Material.FRONT_AND_BACK, color)
        camera.getOrCreateStateSet().setAttributeAndModes(material)
        camera.setClearColor(osg.Vec4(0,0,0,0))
        camera.setClearMask(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

        camera.setCullMask(VISIBLE_CULL_MASK)

        if not self.gw:
            raise Exception("GraphicsWindow not yet created")

        self.camera = camera

#        viewer.getCamera().setViewport(osg.Viewport(0,0, self.width(), self.height()))
 #       viewer.getCamera().addChild(camera)
        camera.setGraphicsContext(self.gw)

    def heightForWidth(self, w):
        ret = int(w * 9.0 / 16.0)
        return ret

    def texture_build(self):
        texture = osg.Texture2D()
        texture.setTextureSize(self.width(), self.height())
        texture.setInternalFormat(GL.GL_RGBA)
        texture.setResizeNonPowerOfTwoHint(False)

        # bug detected here, if I enable mipmap osg seems to use the view buffer to
        # do something. If I disable the mipmap it works.
        # you can view the issue with test_09_gaussian_filter.py
        #texture.setFilter(osg.Texture.MIN_FILTER, osg.Texture.LINEAR_MIPMAP_LINEAR)
        texture.setFilter(osg.Texture.MIN_FILTER, osg.Texture.LINEAR)
        texture.setFilter(osg.Texture.MAG_FILTER, osg.Texture.LINEAR)
        return texture

    def camera_build(self):
        texture = self.texture_build()

        camera = osg.Camera()
        camera.setViewport(osg.Viewport(0,0, self.width(), self.height()))
        camera.setReferenceFrame(osg.Transform.ABSOLUTE_RF)
        camera.setRenderOrder(osg.Camera.PRE_RENDER)
        camera.setRenderTargetImplementation(osg.Camera.FRAME_BUFFER_OBJECT)
        camera.attach(osg.Camera.COLOR_BUFFER, texture, 0, 0, False, 0, 0)

        CAMERA_ANGLE = 45.0
        CAMERA_Z_TRANSLATE = 2.4142135623730949 #1.0 / math.tan(math.radians(CAMERA_ANGLE / 2.0))
        cameraPosition = [0.0, 0.0, CAMERA_Z_TRANSLATE]

        camera.setProjectionMatrixAsPerspective(CAMERA_ANGLE,float(self.width())/float(self.height()), 1.0, 10000.0)

        eye = osg.Vec3d(cameraPosition[0], cameraPosition[1], cameraPosition[2])
        center = osg.Vec3d(0,0,0)
        up = osg.Vec3d(0,1,0)
        camera.setViewMatrixAsLookAt(eye, center, up)

        camera.getOrCreateStateSet().setAttributeAndModes(osg.BlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA))
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_TEST, False)
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_WRITEMASK, False)
        camera.getOrCreateStateSet().setMode(GL.GL_LIGHTING, False)
        material = osg.Material()
        color = osg.Vec4(1.0,1.0,1.0,1.0)
        material.setDiffuse(osg.Material.FRONT_AND_BACK, color)
        material.setAmbient(osg.Material.FRONT_AND_BACK, color)
        camera.getOrCreateStateSet().setAttributeAndModes(material)
        camera.setClearColor(osg.Vec4(0,0,0,0))
        camera.setClearMask(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)


        return camera, texture

    def quad_create(self, texture):
        stateset = osg.StateSet()
        stateset.setTextureAttributeAndModes(0, texture)

        w = 16.0 / 9.0
        h = 1.0
        corner = osg.Vec3(-w,-h, 0)
        width = osg.Vec3(2 * w,0,0)
        height = osg.Vec3(0,2 * h,0)

        geom = osg.createTexturedQuadGeometry(corner, width, height, 0.0, 0.0, 1.0, 1.0)
        geom.setStateSet(stateset)

        geode = osg.Geode()
        geode.addDrawable(geom)
        return geode


    def build_wrapping_node(self, data):
        grp = osg.Group()
        camera,texture = self.camera_build()

        grp.addChild(camera)
        camera.addChild(data)

        quad = self.quad_create(texture)
        grp.addChild(quad)

        grp.getOrCreateStateSet().setMode(GL.GL_LIGHTING, False)
        return grp

    def get_current_time(self):
        if self.mousePressed:
            return self.current_mouse_time
        else:
            return time.time() - self.startTime

    def resetSceneData(self, data):
        self.viewer.setSceneData(None)
        self.audioStop()
        self.audio = None

    def setSceneData(self, data):
        start = time.time()
#        self.startTime = time.time()

        if data  != None:
            data = self.build_wrapping_node(data)
            self.viewer.setSceneData(data)

        end = time.time()
        self.startTime += end - start

    def audioPlay(self):
        if self.audio != None:
            self.audio.play()

    def audioStop(self):
        if self.audio != None:
            self.audio.stop()

    def buildAudioTmpFile(self, data, skip):
        import wavehelper
        if isinstance(data, basestring):
            data = wavehelper.WaveReader(data)
        if hasattr(data, "read"):
            outputFileName = jarvis.get_filename("audiotmpfile.wav")
            ws = wavehelper.WaveSink(data, outputFileName, self.loopTime, skip = skip)
            ws.run()
            return outputFileName

    def setAudioData(self, data, skip = 0.0):
        if self.audio != None:
            self.audioStop()
            self.audio = None
        fname = self.buildAudioTmpFile(data, skip)
        self.audio = QtGui.QSound(fname)
        self.audioPlay()

    def getosgviewer(self):
        return self.viewer

    def resizeGL( self, w, h ):
        self.gw.resized(0,0,w,h)
        self.resetCamera(self.viewer)

    def paintGL (self):
        t = self.get_current_time()
        if t >= self.loopTime:
            self.startTime = time.time()
            t = 0.0
            if self.audio != None:
                self.audioStop()
                self.audioPlay()

        self.viewer.frameAtTime(t)

    def set_time_from_mouse(self, position):
        self.current_mouse_time = self.loopTime * float(position) / float(self.width())
        self.startTime = time.time() - self.current_mouse_time

    def mousePressEvent( self, event ):
        """put the qt event in the osg event queue"""
        button = mouseButtonDictionary.get(event.button(), 0)
        self.mousePressed = True
        self.set_time_from_mouse(event.x())
        self.gw.getEventQueue().mouseButtonPress(event.x(), event.y(), button)
        self.audioStop()

    def mouseReleaseEvent( self, event ):
        """put the qt event in the osg event queue"""
        button = mouseButtonDictionary.get(event.button(), 0)
        self.mousePressed = False
        self.gw.getEventQueue().mouseButtonRelease(event.x(), event.y(), button)

    def mouseMoveEvent(self, event):
        """put the qt event in the osg event queue"""
        self.set_time_from_mouse(event.x())
        self.gw.getEventQueue().mouseMotion(event.x(), event.y())

    def keyPressEvent(self, event):
        """ translate the qt event to an osg event (currently only ascii values) """
        print "key pressed", event.text()
        self.gw.getEventQueue().keyPress( ord(event.text().toAscii().data()) ) #

    def keyReleaseEvent(self, event):
        """ translate the qt event to an osg event (currently only ascii values) """
        print "key released", event.text()
        self.gw.getEventQueue().keyRelease( ord(event.text().toAscii().data()) )

    def timerEvent(self, timerevent):
        """periodically called (each cycle) calls updateGL (which will trigger paintGL)"""
        self.updateGL()

    def getGraphicsWindow(self):
        """returns the osg graphicswindow created by osgViewer """
        return self.gw




from gevent.coros import RLock
import Crypto.Random as CryptoRandom
import base64
import urllib

class LockContext(object):
    """
    Used by :meth:`RedisBase.use_pipe` to temporary substitute a redis client
    with a :class:`redis.client.Pipeline` object.
    """

    def __init__(self, lock):
        self.lock = lock

    def __enter__(self):
        self.lock.acquire()

    def __exit__(self, *exc_info):
        self.lock.release()

class GeventLockable(object):
    def __init__(self):
        self.__lock__ = RLock()
        
    def lock(self):
        return LockContext(self.__lock__)


def generate_random(length):
    p = CryptoRandom.get_random_bytes(length)
    p = base64.b32encode(p)
    return p.strip("=")


def update(base_url, id, op, content):
    body = urllib.urlencode({"op":op, "content":content})
    print body, base_url
    response = urllib.urlopen(base_url + id + "/", body).read()
    print "response=", response
    if response != "OK":
        raise Exception(response)
    



import jarvis
import inspect
import os
import time
import jarvis.client as client

def set_test_filename_function(test_filename_function):
    """Set the module.function that jarvis watches for changes and then executes."""
    # Get the jarvis home directory
    jarvis_home = jarvis.get_home()

    # Find the file that holds the current module.function to test
    testmodulefilename = os.path.join(jarvis_home, jarvis.TEST_FILENAME_FUNCTION)

    # Write the file
    f=open(testmodulefilename, "w")
    f.write(test_filename_function)
    f.close()

set_test_filename_function.__test__ = False

start_time = time.time()
def delta_time_format():
    global start_time
    return str(time.time() - start_time)[:6] + "s"

def error(*args):
    if jarvis.ml != None and jarvis.ml.display != None:
        jarvis.ml.display.errorprint(delta_time_format(), *args)
    print " ".join(args)

def reset_start_time():
    global start_time
    start_time = time.time()

def display_reset():
    # TEMPORARY
    client.Client().update("debug", "set", "")

def debug(*args):
    msg = " ".join(map(lambda x : str(x), args))

    if jarvis.ml != None and jarvis.ml.display != None:
        jarvis.ml.display.debugprint(delta_time_format(), *args)

#    print client.Client().update("debug", "append", msg + "<br/>")

def debug_dir(object, filt = None):
    debug("debug_dir", object, filt)
    for k in dir(object):
        if filt == None or filt in k.lower():
            debug(k)
def debug_xml(*args):
    import lxml
    debug(*(list(args[:-1]) + [lxml.etree.tostring(args[-1], pretty_print = True)]))

def debug_osg(osgdata):
    if jarvis.ml != None and jarvis.ml.display != None:
        jarvis.ml.display.osgprint(osgdata)

def debug_audio(audiodata, skip = 0.0):
    if jarvis.ml != None and jarvis.ml.display != None:
        jarvis.ml.display.audioemit(audiodata, skip)

def debug_osg_set_loop_time(loop_time):
    if jarvis.ml != None and jarvis.ml.display != None:
        jarvis.ml.display.setlooptime(loop_time)

def testunit_result(result):
    for err in result.errors:
        error(err[1])
    for err in result.failures:
        error(err[1])

testunit_result.__test__ = False

def get_osg_viewer():
    if jarvis.ml != None and jarvis.ml.display != None:
        return jarvis.ml.display.getosgviewer()

def add_watch_file(filename):
    if jarvis.ml != None:
        return jarvis.ml.add_watch_file(filename)

def replace_this(*args):
    if len(args) == 0:
        raise Exception("jarvis.commands.replace_this: Nothing to replace by")
    # Retrieve the calling frame
    frame = inspect.stack()[1]
    # Get the file name and the line_number
    file_name = frame[1]
    line_number = frame[2]
    # Split by line the source file
    lines = open(file_name).read().split("\n")
    # Retrieve the right line
    line = lines[line_number - 1]
    # Check that the source code is still matching
    inspect_line = frame[4][0][:-1]
    if line != inspect_line or "replace_this" not in line:
        raise Exception("jarvis.commands.replace_this: looks like the file changed before replace could occur.")

    for i,c in enumerate(inspect_line):
        if c != " ":
            break
    # Build the replacement line
    replacement = " " * i + args[0] % args[1:]
    # Replace the line
    lines[line_number - 1] = replacement
    # Rebuild the file content
    file_content = "\n".join(lines)
    # Write back the file
    file = open(file_name, "w")
    file.write(file_content)
    file.close()


MAX_INSPECT_ELAPSED = 10
def external_inspect_vars(file_name, line_number):
    filename = jarvis.get_filename(jarvis.INSPECT_VAR_QUERY)
    f = open(filename, "w")
    query = "%s %s" % (line_number, file_name)
    f.write(query)
    f.close()
    start = time.time()
    elapsed = 0
    while elapsed < MAX_INSPECT_ELAPSED:
        try:
            filename = jarvis.get_filename(jarvis.INSPECT_VAR_QUERY_RESPONSE)
            r = open(filename).read()
            if r.split("\n")[0] == query:
                return r
        except IOError:
            pass

        elapsed = time.time() - start


